# Pestifer
> Automatic NAMD System Input Builder

Pestifer is a Python utility for generating atomistic inputs with appropriate topology and parameter files required for molecular dynamics simulations using NAMD.  It is intended as a fully automated system builder requiring as inputs only the PDB codes of biomolecular structures and a handful of customization parameters.  Pestifer primarily works as a front end for VMD's `psfgen` utility.

## Installation

```bash
pip install pestifer
```

Once installed, the user has access to the main `pestifer` command.

## Release History
* 1.3.7
    * bugfix:
      * fixed a spurious hard-coded path in `macros.tcl`
* 1.3.6
    * bugfix:
      * `runscript` sources TcL proc files with dependencies in proc files that aren't yet sourced; fixed that
    * `alphafold` source directive added to permit download of models from the AlphaFold database by accession code
* 1.3.5
    * bugfix:
      * renumbering of author resids in non-protein segments if user adds protein residues by insertion that may conflict
    * transferance of atomselect macros from YAML input to any VMD script
    * `inittcl` subcommand makes this transfer; needs only to be run one time post-installation
* 1.3.4
    * new TcL procs for asymmetric unit generation from non-symmetric assemblies
    * `pestifer_init` TcL proc provided in docs for user VMD startup script
    * `script` subcommand removed
    * syntax of `wheretcl` subcommand expanded
* 1.3.3
    * `NAMDLog` class introduced for parsing NAMD2-generated log files
    * `mdplot` task for generating plots of various energy-like quantities vs timestep
* 1.3.2
    * allow for user-defined links in the config file
    * all example builds now have tests in the test suite
* 1.3.1
    * bug fixes for cleaving
* 1.3.0
    * Support for reading from already-build PSF/PDB systems
* 1.2.9
    * improved declashing and domain-swapping
* 1.2.8
    * `grafts` for adding glycans
* 1.2.7
    * `cleave` task and `CleavageMod`
* 1.2.6
    * `ModManager` replaces `ModContainer`
* 1.2.5
    * `insertion` mod; corrected bug in `brot` tcl procedure
* 1.2.3
    * script subcommand handles local scripts
* 1.2.2
    * added `wheretcl` subcommand
* 1.2.1
    * added `script` subcommand
* 1.2.0
    * split all namd subtasks out; now they are level-1 tasks
* 1.1.3
    * added `manipulate` task
* 1.1.2
    * more control over production NAMD2 config generated by the package directive
* 1.1.1
    * position restraints control in minimization and relaxation
* 1.1.0
    * `other_parameters` for any NAMD2 relaxation task
* 1.0.9
    * alternate coordinate files and Cfusions
* 1.0.8
    * chain-specific control over building in zero-occupancy residues at N and C termini
* 1.0.7
    * `alpha` crotation for folding a span of residues into an alpha helix
* 1.0.6
    * `cif_residue_map_file` generated to report mapping between CIF-residue numbering and author residue numbering
* 1.0.5
    * enhancements to packaging task
* 1.0.4
    * support for topogromacs added
* 1.0.1
    * Initial version

## Meta

Cameron F. Abrams

Distributed under the MIT license. See ``LICENSE`` for more information.

[https://github.com/cameronabrams](https://github.com/cameronabrams/)

## Contributing

1. Fork it (<https://github.com/cameronabrams/pestifer/fork>)
2. Create your feature branch (`git checkout -b feature/fooBar`)
3. Commit your changes (`git commit -am 'Add some fooBar'`)
4. Push to the branch (`git push origin feature/fooBar`)
5. Create a new Pull Request

